@@ -1,7 +1,8 @@
 from extrack import tracking
+from extrack import old_tracking
 from extrack.version import __version__
 print('version:', __version__)
-from extrack import auto_fitting
+#from extrack import auto_fitting
 from extrack import exporters
 from extrack import histograms
 from extrack import readers
